from fundrive.lanzou import LanZouCloud
from funtool.log import logger
from funtool.tool.secret import read_secret

downer = LanZouCloud()
downer.ignore_limits()
downer.login_by_cookie()

logger.info("begin")
# print(downer.sync_directory(path_root='/root/workspace/tmp/coin', folder_id='3358325'))
downer.upload_file("/root/workspace/tmp/coin/dbs/huobi-SHIBUSDT.db", folder_id="3907901")



import argparse
from datetime import datetime

from ccxt import binance
from funcoin.base.drive.lanzou import LanzouDirectory
from funcoin.coins.base.file import DataFileProperty
from funcoin.task.load import LoadTask


def download(args):
    file_pro = DataFileProperty(exchange=binance(), path="tmp")
    file_pro.file_format = "%Y%m%d"
    file_pro.change_data_type(args.type)
    file_pro.change_timeframe(args.timeframe)
    file_pro.change_freq(args.freq)
    file_pro.load_days()


def sync(args):
    lanzou = LanzouDirectory(fid=6073401)
    lanzou.scan_all_file(clear=False)


def funcoin():
    parser = argparse.ArgumentParser(prog="PROG")
    subparsers = parser.add_subparsers(help="sub-command help")
    # 添加子命令
    download_parser = subparsers.add_parser("download", help="download data")
    download_parser.add_argument(
        "--type",
        nargs=2,
        metavar=("kline", "trade"),
        default="kline",
        help="What kind of data do you want to download",
    )

    download_parser.add_argument("-ds", default=None, help="the date")
    download_parser.add_argument("-days", default=7, type=int, help="the date")
    download_parser.add_argument("--freq", nargs=2, metavar=("daily", "dailys"), default="daily", help="")
    download_parser.add_argument("--timeframe", nargs=2, metavar=("1m", "5m"), default="1m", help="")
    download_parser.set_defaults(func=download)  # 设置默认函数

    # 添加子命令
    parser_s = subparsers.add_parser("sync", help="sub help")
    parser_s.set_defaults(func=sync)  # 设置默认函数
    args = parser.parse_args()
    args.func(args)



from funcoin.huobi.connection import (RestApiSyncClient, SubscribeClient,
                                       WebSocketReqClient)
from funcoin.huobi.constant import HttpMethod
from funcoin.huobi.utils import check_should_not_none, check_symbol


class EtfClient(object):

    def __init__(self, *args, **kwargs):
        """
        Create the request client instance.
        :param kwargs: The option of request connection.
            api_key: The public key applied from Huobi.
            secret_key: The private key applied from Huobi.
            url: The URL name like "https://api.huobi.pro".
            init_log: to init logger
        """
        self.__kwargs = kwargs
        self.rest_api_sync_client = RestApiSyncClient(*args, **kwargs)
        self.web_socket_req_client = WebSocketReqClient(*args, **kwargs)
        self.sub_socket_req_client = SubscribeClient(*args, **kwargs)

    def get_etf_swap_config(self, etf_name: 'str'):
        """
        Get the basic information of ETF creation and redemption, as well as ETF constituents,
        including max amount of creation, min amount of creation, max amount of redemption, min amount
        of redemption, creation fee rate, redemption fee rate, eft create/redeem status.

        :param etf_name: The symbol, currently only support hb10. (mandatory)
        :return: The etf configuration information.
        """
        check_symbol(etf_name)
        params = {
            "etf_name": etf_name
        }
        channel = "/etf/swap/config"
        return self.rest_api_sync_client.request_process(HttpMethod.GET, channel, params)

    def get_etf_swap_list(self, etf_name: 'str', offset: 'int', size: 'int') -> list:
        """
        Get past creation and redemption.(up to 100 records)

        :param etf_name: The symbol, currently only support hb10. (mandatory)
        :param offset: The offset of the records, set to 0 for the latest records. (mandatory)
        :param size: The number of records to return, the range is [1, 100]. (mandatory)
        :return: The swap history.
        """
        check_symbol(etf_name)
        params = {
            "etf_name": etf_name,
            "offset": offset,
            "limit": size
        }
        channel = "/etf/swap/list"
        return self.rest_api_sync_client.request_process(HttpMethod.GET_SIGN, channel, params)

    def post_etf_swap_in(self, etf_name: 'str', amount: 'int') -> None:
        """
        Order creation or redemption of ETF.

        :param etf_name: The symbol, currently only support hb10. (mandatory)
        :param amount: The amount to create or redemption. (mandatory)
        :return: No return
        """
        check_symbol(etf_name)
        check_should_not_none(amount, "amount")

        params = {
            "etf_name": etf_name,
            "amount": amount
        }
        channel = "/etf/swap/in"
        return self.rest_api_sync_client.request_process(HttpMethod.POST_SIGN, channel, params)

    def post_etf_swap_out(self, etf_name: 'str', amount: 'int') -> None:
        """
        Order creation or redemption of ETF.

        :param etf_name: The symbol, currently only support hb10. (mandatory)
        :param amount: The amount to create or redemption. (mandatory)
        :return: No return
        """

        check_symbol(etf_name)
        check_should_not_none(amount, "amount")

        params = {
            "etf_name": etf_name,
            "amount": amount
        }
        channel = "/etf/swap/out"
        return self.rest_api_sync_client.request_process(HttpMethod.POST_SIGN, channel, params)



class BaseChannel:
    def __init__(self, channel=None, *args, **kwargs):
        """
        :param channel: 频道名
        """
        self.json = {"channel": channel}

    def to_json(self):
        return self.json

    def put(self, key, value):
        if value is not None:
            self.json[key] = value


class PublicChannel(BaseChannel):
    def __init__(self, instType=None, instId=None, uly=None, *args, **kwargs):
        """
        :param instType: 产品类型
        :param instId: 产品ID
        :param uly: 合约标的指数
        """
        super(PublicChannel, self).__init__(*args, **kwargs)
        self.put("instType", instType)
        self.put("instId", instId)
        self.put("uly", uly)

    @staticmethod
    def public_instruments(instType="FUTURES"):
        """
        产品频道
        """
        return PublicChannel(channel="instruments", instType=instType)

    @staticmethod
    def public_tickers(instId="BTC-USDT"):
        """
        行情频道
        """
        return PublicChannel(channel="tickers", instId=instId)

    @staticmethod
    def public_open_interest(instId="BTC-USDT"):
        """
        持仓总量频道
        """
        return PublicChannel(channel="open-interest", instId=instId)

    @staticmethod
    def public_kline(channel="candle1m", instId="BTC-USDT"):
        """
        K线频道
        candle1Y
        candle6M candle3M candle1M
        candle1W
        candle1D candle2D candle3D candle5D
        candle12H candle6H candle4H candle2H candle1H
        candle30m candle15m candle5m candle3m candle1m
        candle1Yutc candle3Mutc candle1Mutc candle1Wutc candle1Dutc candle2Dutc candle3Dutc candle5Dutc candle12Hutc candle6Hutc
        """
        return PublicChannel(channel=channel, instId=instId)

    @staticmethod
    def public_trades(instId="BTC-USDT"):
        """
        交易频道
        """
        return PublicChannel(channel="trades", instId=instId)

    @staticmethod
    def public_estimated_price(instType='FUTURES', uly="BTC-USDT"):
        """
        预估交割/行权价格频道
        """
        return PublicChannel(channel="estimated-price", instType=instType, uly=uly)

    @staticmethod
    def public_mark_price(instId="BTC-USDT"):
        """
        标记价格频道
        """
        return PublicChannel(channel="mark-price", instId=instId)

    @staticmethod
    def public_mark_price_kline(channel="", instId="BTC-USDT"):
        """
        标记价格K线频道
        mark-price-candle1Y
        mark-price-candle6M
        mark-price-candle3M
        mark-price-candle1M
        mark-price-candle1W
        mark-price-candle1D
        mark-price-candle2D
        mark-price-candle3D
        mark-price-candle5D
        mark-price-candle12H
        mark-price-candle6H
        mark-price-candle4H
        mark-price-candle2H
        mark-price-candle1H
        mark-price-candle30m
        mark-price-candle15m
        mark-price-candle5m
        mark-price-candle3m
        mark-price-candle1m
        mark-price-candle1Yutc
        mark-price-candle3Mutc
        mark-price-candle1Mutc
        mark-price-candle1Wutc
        mark-price-candle1Dutc
        mark-price-candle2Dutc
        mark-price-candle3Dutc
        mark-price-candle5Dutc
        mark-price-candle12Hutc
        mark-price-candle6Hutc
        """
        return PublicChannel(channel=channel, instId=instId)

    @staticmethod
    def public_price_limit(instId="BTC-USDT"):
        """
        限价频道
        """
        return PublicChannel(channel="price-limit", instId=instId)

    @staticmethod
    def public_books(instId="BTC-USDT"):
        """
        深度频道
        """
        return PublicChannel(channel="books", instId=instId)

    @staticmethod
    def public_opt_summary(uly="BTC-USDT"):
        """
        期权定价频道
        """
        return PublicChannel(channel="opt-summary", uly=uly)

    @staticmethod
    def public_funding_rate(instId="BTC-USDT"):
        """
        资金费率频道
        """
        return PublicChannel(channel="funding-rate", instId=instId)

    @staticmethod
    def public_index_kline(channel, instId="BTC-USDT"):
        """
        指数K线频道
        index-candle1Y
        index-candle6M
        index-candle3M
        index-candle1M
        index-candle1W
        index-candle1D
        index-candle2D
        index-candle3D
        index-candle5D
        index-candle12H
        index-candle6H
        index-candle4H
        index -candle2H
        index-candle1H
        index-candle30m
        index-candle15m
        index-candle5m
        index-candle3m
        index-candle1m
        index-candle1Yutc
        index-candle3Mutc
        index-candle1Mutc
        index-candle1Wutc
        index-candle1Dutc
        index-candle2Dutc
        index-candle3Dutc
        index-candle5Dutc
        index-candle12Hutc
        index-candle6Hutc
        """
        return PublicChannel(channel=channel, instId=instId)

    @staticmethod
    def public_index_tickers(instId="BTC-USDT"):
        """
        指数行情频道
        """
        return PublicChannel(channel="index-tickers", instId=instId)

    @staticmethod
    def public_status():
        """
        指数行情频道
        """
        return PublicChannel(channel="status")


class PrivateChannel(BaseChannel):
    def __init__(self, ccy=None, instType=None, uly=None, instId=None, *args, **kwargs):
        """
        :param instType: 产品类型
        :param instId: 产品ID
        :param uly: 合约标的指数
        :param ccy: 币种
        """
        super(PrivateChannel, self).__init__(*args, **kwargs)
        self.put("instType", instType)
        self.put("instId", instId)
        self.put("ccy", ccy)
        self.put("uly", uly)

    @staticmethod
    def private_instruments(ccy="BTC"):
        """
        账户频道
        """
        return PrivateChannel(channel="account", ccy=ccy)

    @staticmethod
    def private_positions(instType="FUTURES", uly="BTC-USDT", instId="BTC-USDT"):
        """
        持仓频道
        """
        return PrivateChannel(channel="positions", instType=instType, uly=uly, instId=instId)

    @staticmethod
    def private_orders(instType="FUTURES", uly="BTC-USDT", instId="BTC-USDT"):
        """
        持仓频道
        """
        return PrivateChannel(channel="orders", instType=instType, uly=uly, instId=instId)

    @staticmethod
    def private_orders_algo(instType="FUTURES", uly="BTC-USDT", instId="BTC-USDT"):
        """
        持仓频道
        """
        return PrivateChannel(channel="orders-algo", instType=instType, uly=uly, instId=instId)


'''
交易 trade
'''

# 下单
# trade_param = {"id": "1512", "op": "order", "args": [{"side": "buy", "instId": "BTC-USDT", "tdMode": "isolated", "ordType": "limit", "px": "19777", "sz": "1"}]}
# 批量下单
# trade_param = {"id": "1512", "op": "batch-orders", "args": [
#         {"side": "buy", "instId": "BTC-USDT", "tdMode": "isolated", "ordType": "limit", "px": "19666", "sz": "1"},
#         {"side": "buy", "instId": "BTC-USDT", "tdMode": "isolated", "ordType": "limit", "px": "19633", "sz": "1"}
#     ]}
# 撤单
# trade_param = {"id": "1512", "op": "cancel-order", "args": [{"instId": "BTC-USDT", "ordId": "259424589042823169"}]}
# 批量撤单
# trade_param = {"id": "1512", "op": "batch-cancel-orders", "args": [
#         {"instId": "BTC-USDT", "ordId": "259432098826694656"},
#         {"instId": "BTC-USDT", "ordId": "259432098826694658"}
#     ]}
# 改单
# trade_param = {"id": "1512", "op": "amend-order", "args": [{"instId": "BTC-USDT", "ordId": "259432767558135808", "newSz": "2"}]}
# 批量改单
# trade_param = {"id": "1512", "op": "batch-amend-orders", "args": [
#         {"instId": "BTC-USDT", "ordId": "259435442492289024", "newSz": "2"},
#         {"instId": "BTC-USDT", "ordId": "259435442496483328", "newSz": "3"}
#     ]}



import os
from datetime import datetime, timedelta

import pandas as pd
from funcoin.huobi.history.core import ALL_PERIODS, ALL_TYPES, load_daily_all
from fundrive.lanzou import LanZouCloud
from noteodps import opt
from odps import DataFrame
from tqdm import tqdm


def save_to_lanzou():
    downer = LanZouCloud()
    downer.ignore_limits()
    downer.login_by_cookie()
    start_date = datetime(2021, 5, 3)
    end_date = datetime(2021, 5, 31)
    duration = end_date - start_date
    for i in range(duration.days + 1):
        day = end_date - timedelta(days=i)
        file_path = load_daily_all(period="1min", date=day)
        downer.upload_file(file_path, folder_id="3359096")


def save_kline_to_odps(start_date=datetime(2021, 1, 1), end_date=datetime(2021, 5, 31)):
    t = opt.get_table("ods_funcoin_huobi_klines_data_d")

    def save_file_to_odps(file_path, partition):
        if t.exist_partition(partition):
            print(f"{partition} exist,drop it.")
            return
            t.delete_partition(partition_spec=partition)
        columns = ["symbol", "id", "open", "close", "low", "high", "vol", "amount"]
        dtype = {
            "symbol": "str",
            "id": "long",
            "open": "float",
            "close": "float",
            "low": "float",
            "high": "float",
            "vol": "float",
            "amount": "float",
        }
        with t.open_writer(partition=partition, create_partition=True) as writer:
            for df in tqdm(
                pd.read_csv(file_path, header=None, names=columns, dtype=dtype, chunksize=300000),
                desc=os.path.basename(file_path),
            ):
                writer.write(df.values.tolist())

    duration = end_date - start_date
    for period in ["1min"]:  # ALL_PERIODS:
        for _type in ["future"]:  # ALL_TYPES:
            for i in range(duration.days + 1):
                day = end_date - timedelta(days=i)
                partition = f"type='{_type}',period='{period}',ds='{day.strftime('%Y%m%d')}'"
                if t.exist_partition(partition):
                    continue

                file_path = load_daily_all(period=period, _type=_type, date=day)
                save_file_to_odps(file_path, partition)


def save_trade_to_odps(start_date=datetime(2021, 1, 1), end_date=datetime(2021, 5, 31)):
    t = opt.get_table("ods_funcoin_huobi_trades_data_d")

    def save_file_to_odps(file_path, partition):
        if t.exist_partition(partition):
            print(f"{partition} exist,drop it.")
            return
            t.delete_partition(partition_spec=partition)
        columns = ["symbol", "tid", "ts", "price", "amount", "quantity", "direction"]
        dtype = {
            "symbol": "str",
            "tid": "long",
            "ts": "long",
            "price": "float",
            "amount": "float",
            "quantity": "float",
            "direction": "str",
        }
        with t.open_writer(partition=partition, create_partition=True) as writer:
            for df in tqdm(
                pd.read_csv(file_path, header=None, names=columns, dtype=dtype, chunksize=300000),
                desc=os.path.basename(file_path),
            ):
                writer.write(df.values.tolist())

    duration = end_date - start_date
    for period in ["1min"]:  # ALL_PERIODS:
        for _type in ["future"]:  # ALL_TYPES:
            for i in range(duration.days + 1):
                day = end_date - timedelta(days=i)
                partition = f"type='{_type}',period='{period}',ds='{day.strftime('%Y%m%d')}'"
                if t.exist_partition(partition):
                    continue

                file_path = load_daily_all(period=period, _type=_type, date=day, trade=True)
                save_file_to_odps(file_path, partition)


save_kline_to_odps(start_date=datetime(2021, 1, 1), end_date=datetime(2021, 10, 24))
save_trade_to_odps(start_date=datetime(2021, 1, 1), end_date=datetime(2021, 10, 24))



import copy
import logging
import os
from datetime import datetime, timedelta

from funcoin.base.drive.lanzou import LanzouDirectory
from funcoin.coins.base.load import LoadDataKline, LoadTradeKline
from funfile.compress import tarfile
from tqdm import tqdm

logger = logging.getLogger()


def parse_day(days=0):
    first = datetime.now() - timedelta(days=days)
    last = first + timedelta(days=1) - timedelta(seconds=1)
    return first, last


def merge_unique(input_csv_list, put_csv):
    with open(put_csv, "w") as outfile:
        for i, csv_file in tqdm(enumerate(input_csv_list), desc="merge files", total=len(input_csv_list)):
            with open(csv_file, "r") as infile:
                if i > 0:
                    next(infile)
                for line in infile:
                    outfile.write(line)


class FileProperty:
    def __init__(
        self,
        exchange_name="okex",
        data_type="kline",
        path="~/workspace/tmp",
        start_date=datetime.today(),
        end_date=datetime.today(),
        freq="daily",
        timeframe="1m",
        file_format="%Y%m%d",
    ):
        self.path = path
        self.freq = freq
        self.data_type = data_type
        self.timeframe = timeframe
        self.start_date = start_date
        self.end_date = end_date
        self.file_format = file_format
        self.exchange_name = exchange_name

    def file_path_dir(self, absolute=True):
        path = f"funcoin/{self.exchange_name}/{self.data_type}-{self.freq}-{self.timeframe}"
        if absolute:
            path = f"{self.path}/{path}"
            if not os.path.exists(path):
                os.makedirs(path)
        return path

    @property
    def filename_prefix(self):
        return f"{self.exchange_name}-{self.data_type}-{self.freq}-{self.timeframe}-{self.start_date.strftime(self.file_format)}"

    def file_path_csv(self, absolute=True):
        return f"{self.file_path_dir(absolute)}/{self.filename_prefix}.csv"

    def file_path_tar(self, absolute=True):
        return f"{self.file_path_dir(absolute)}/{self.filename_prefix}.tar"

    def arcname(self, file):
        return os.path.join(self.file_path_dir(False), os.path.basename(file))


class DataFileProperty:
    def __init__(
        self,
        exchange,
        data_type="kline",
        path="tmp",
        start_date=datetime.today(),
        end_date=datetime.today(),
        freq="daily",
        timeframe="1m",
        file_format="%Y%m%d",
        download_days=31,
    ):
        self.exchange = exchange
        self.data_type = data_type
        self.file_pro = FileProperty(
            exchange_name=exchange.name.lower(),
            path=path,
            freq=freq,
            data_type=data_type,
            timeframe=timeframe,
            start_date=start_date,
            end_date=end_date,
            file_format=file_format,
        )
        self.drive = LanzouDirectory(fid=6073401)
        self.download_days = download_days
        if not os.path.exists(path):
            os.makedirs(path)
        if not os.path.exists(f"{path}/funcoin"):
            os.makedirs(f"{path}/funcoin")
        print("cwd : " + os.getcwd())

    def change_freq(self, freq):
        self.file_pro.freq = freq

    def change_data_type(self, data_type):
        self.data_type = data_type
        self.file_pro.data_type = data_type

    def change_timeframe(self, timeframe):
        self.file_pro.timeframe = timeframe

    def sync(self, path):
        logger.info("sync file")
        self.drive.scan_all_file()
        self.drive.sync(f"{path}/funcoin", remove_local=True)

    def tar_exists(self):
        if self.drive.file_exist(self.file_pro.file_path_tar(absolute=False)):
            return True
        return False

    def _daily_load_and_save(self, file_pro: FileProperty, check_sync=True) -> bool:
        if self.tar_exists():
            return False
        if check_sync:
            self.sync(self.file_pro.path)

        logger.info(f"download for {file_pro.file_path_tar(absolute=False)}")
        unix_start, unix_end = int(file_pro.start_date.timestamp() * 1000), int(file_pro.end_date.timestamp() * 1000)

        if not os.path.exists(self.file_pro.file_path_tar()):
            if not os.path.exists(self.file_pro.file_path_csv()):
                self.download_days -= 1
                if self.data_type == "kline":
                    exchan = LoadDataKline(
                        self.exchange,
                        unix_start=unix_start,
                        unix_end=unix_end,
                        csv_path=file_pro.file_path_csv(),
                        timeframe=file_pro.timeframe,
                    )
                else:
                    exchan = LoadTradeKline(
                        self.exchange, unix_start=unix_start, unix_end=unix_end, csv_path=file_pro.file_path_csv()
                    )
                # 下载
                exchan.load_all()
            # 压缩
            with tarfile.open(file_pro.file_path_tar(), "w|xz") as tar:
                tar.add(file_pro.file_path_csv(), arcname=file_pro.arcname(file_pro.file_path_csv()))
        # 删除
        if os.path.exists(file_pro.file_path_csv()):
            os.remove(file_pro.file_path_csv())
        return True

    def _merge_and_save(self, file_pro: FileProperty) -> bool:
        if self.tar_exists():
            return False
        self.sync(self.file_pro.path)

        logger.info(f"merge for {file_pro.file_path_tar(absolute=False)}")

        result = []
        for i in range(1000):
            day = copy.deepcopy(file_pro)
            day.freq = "daily"
            day.start_date, day.end_date = file_pro.start_date + timedelta(days=i), file_pro.start_date + timedelta(
                days=i + 1
            )
            if day.start_date < file_pro.start_date:
                continue
            if day.start_date > file_pro.end_date:
                break
            result.append(day.file_path_csv())
            if os.path.exists(day.file_path_csv()):
                continue
            # download
            fid = self.drive.file_fid(day.file_path_tar(False))
            if fid == -1:
                self._daily_load_and_save(day)
                fid = self.drive.file_fid(day.file_path_tar(False))

            self.drive.drive.down_file_by_id(fid, save_path=os.path.dirname(day.file_path_tar()), overwrite=True)
            # unzip
            with tarfile.open(day.file_path_tar(), "r|xz") as tar:
                tar.extractall(path=day.path)
            os.remove(day.file_path_tar())
            # remove
        merge_unique(result, file_pro.file_path_csv())

        # 压缩
        with tarfile.open(file_pro.file_path_tar(), "w|xz") as tar:
            tar.add(file_pro.file_path_csv(), arcname=file_pro.arcname(file_pro.file_path_csv()))
        # 删除
        os.remove(file_pro.file_path_csv())
        [os.remove(csv) for csv in result]
        return True

    def load_days(self):
        for i in range(1, 365):
            start_time, end_time = parse_day(i)
            self.file_pro.start_date, self.file_pro.end_date = start_time, end_time
            self._daily_load_and_save(self.file_pro)
            if self.download_days < 0:
                return False

    def load_daily(self, start_time, end_time):
        self.file_pro.start_date, self.file_pro.end_date = start_time, end_time
        self._daily_load_and_save(self.file_pro)

    def load_merge(self, start_time, end_time):
        self.file_pro.start_date, self.file_pro.end_date = start_time, end_time
        self._merge_and_save(self.file_pro)



import ccxt
from funcoin.coins.base.cron import load_month
from funsecret.secret import read_secret, write_secret
from funsecret import read_secret

path_root = read_secret(cate1="funcoin", cate2="path", cate3="local", cate4="path_root") or "~/workspace/tmp"

load_month(ccxt.okex(), tmp_path=path_root)



import os

from funcoin.base.db import BaseTable, Base
from fundrive.lanzou import LanZouDrive
from sqlalchemy import BIGINT, String, select, UniqueConstraint, DateTime, func
from sqlalchemy.orm import mapped_column
from tqdm import tqdm


class FunCoinLanZou(Base):
    __tablename__ = "funcoin_lanzou"
    __table_args__ = (UniqueConstraint("fid"),)

    fid = mapped_column(BIGINT, comment="fid", primary_key=True)
    gmt_create = mapped_column(DateTime(timezone=True), server_default=func.now())
    gmt_modified = mapped_column(DateTime(timezone=True), server_default=func.now(), onupdate=func.now())
    isfile = mapped_column(BIGINT, comment="是否文件", default="1")
    name = mapped_column(String(100), comment="名称", default="")
    desc = mapped_column(String(500), comment="描述", default="0")
    path = mapped_column(String(500), comment="路径", default="")
    downs = mapped_column(BIGINT, comment="下载次数", default="0")
    url = mapped_column(String(100), comment="分享链接", default="0")
    pwd = mapped_column(String(50), comment="提取码", default="0")


class LanzouDirectory(BaseTable):
    def __init__(self, fid=5679873, *args, **kwargs):
        self.fid = fid
        super(LanzouDirectory, self).__init__(table=FunCoinLanZou, *args, **kwargs)
        self.drive = LanZouDrive()
        self.drive.login()
        self.drive.ignore_limit()

    def scan_all_file(self, clear=False):
        if clear:
            self.delete_all()
        self._scan_all_file(self.fid, "funcoin")

    def _scan_all_file(self, fid, path):
        for _dir in self.drive.get_dir_list(fid):
            _path = f"{path}/{_dir['name']}"
            if not self.file_exist(_path):
                share = self.drive.get_file_info(fid=_dir['fid'], is_file=False)
                data = {
                    "fid": _dir['fid'],
                    "name": _dir['name'],
                    "path": _path,
                    "isfile": 0,
                    "desc": share['desc'],
                    "url": share['url'],
                    "pwd": share['pwd'],
                }
                self.upsert(values=data)

            if _dir['name'].endswith(".tar"):
                continue
            self._scan_all_file(fid=_dir['fid'], path=_path)

        for _file in tqdm(self.drive.get_file_list(fid), desc=f"fid={fid}-{os.path.basename(path)}"):
            _path = f"{path}/{_file['name']}"

            if not self.file_exist(_path):
                share = self.drive.get_file_info(fid=_file['fid'], is_file=True)
                data = {
                    "fid": _file['fid'],
                    "name": _file['name'],
                    "path": _path,
                    "isfile": 1,
                    "downs": _file['downs'],
                    "desc": share['desc'],
                    "url": share['url'],
                    "pwd": share['pwd'],
                }
                self.upsert(values=data)

    def file_exist(self, path):
        s = select(self.table).where(FunCoinLanZou.path == path)
        with self.engine.connect() as conn:
            data = [line for line in conn.execute(s)]
            return len(data) == 1

    def file_fid(self, path):
        s = select(self.table).where(FunCoinLanZou.path == path)
        with self.engine.connect() as conn:
            data = [line for line in conn.execute(s)]
            if len(data) == 1:
                return data[0]["fid"]
            else:
                return -1

    def sync(self, path, remove_local=True, *args, **kwargs):
        def filter_fun(x):
            return x.endswith(".csv") or x.startswith("_")

        self.drive.upload_dir(path, self.fid, remove_local=remove_local, filter_fun=filter_fun)



from funcoin.base.db.mysql import BaseTable
from sqlalchemy import BIGINT, Column, Float, String, Table, func, select


class KlineData(BaseTable):
    def __init__(self, table_name="kline_data_1min", prefix="com", suffix="com", *args, **kwargs):
        table_name = f"{prefix.lower()}_{table_name}_{suffix.lower()}"

        super(KlineData, self).__init__(table_name=table_name, db_suffix=prefix.lower(), *args, **kwargs)
        self.table = Table(
            self.table_name,
            self.meta,
            Column("symbol", String(30), comment="symbol", primary_key=True),
            Column("timestamp", BIGINT, comment="timestamp", primary_key=True),
            Column("open", Float, comment="open", default=0),
            Column("close", Float, comment="close", default=0),
            Column("low", Float, comment="low", default=0),
            Column("high", Float, comment="high", default=0),
            Column("vol", Float, comment="vol", default=0),
            extend_existing=True,
            # autoload=True
        )
        self.create()

    def select_symbol_maxmin(self, symbol):
        self.create()
        s = (
            select(
                self.table.columns.symbol,
                func.max(self.table.columns.timestamp).label("max_time"),
                func.min(self.table.columns.timestamp).label("min_time"),
            )
            .where(self.table.columns.symbol == symbol)
            .group_by(self.table.columns.symbol)
        )
        with self.engine.connect() as conn:
            data = [line for line in conn.execute(s)]
            if len(data) == 1:
                return data[0][1], data[0][2]
            else:
                return 0, 0



from funcoin.huobi.client import GenericClient, MarketClient
from funcoin.huobi.dataset.core import *
from funcoin.huobi.history.core import HistoryDownload
from funtool.tool.secret import read_secret
from tqdm import tqdm


class TradeUpdateHistory:
    def __init__(self, db_path="/root/workspace/tmp/coin/huobi.db", date="20210101"):
        db_path = db_path.replace(".db", f"-{date}.db")
        api_key = read_secret(cate1="coin", cate2="huobi", cate3="api_key")
        secret_key = read_secret(cate1="coin", cate2="huobi", cate3="secret_key")
        self.market = MarketClient(api_key=api_key, secret_key=secret_key)
        self.generic = GenericClient(api_key=api_key, secret_key=secret_key)

        self.symbolDB = SymbolInfo(db_path=db_path)
        self.tradeDB = TradeDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_1min = Kline1MinDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_5min = Kline5MinDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_15min = Kline15MinDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_30min = Kline30MinDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_60min = Kline60MinDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_4hour = Kline4HourDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_1day = Kline1DayDetail(db_path=db_path, conn=self.symbolDB.conn)

        self.init()

    def init(self):
        self.symbolDB.create()
        self.tradeDB.create()
        self.kline_1min.create()
        self.kline_5min.create()
        self.kline_15min.create()
        self.kline_30min.create()
        self.kline_60min.create()
        self.kline_4hour.create()
        self.kline_1day.create()

    @staticmethod
    def replace_key(data_dicts: list, symbol=None):
        res = []
        for _data_dict in data_dicts:
            data_dict = {"symbol": symbol}
            for key in _data_dict.keys():
                data_dict[key.replace("-", "_")] = _data_dict[key]
            res.append(data_dict)
        return res

    def insert_symbol(self):
        self.symbolDB.insert_list(self.replace_key(self.generic.get_exchange_symbols().data))

    def insert_trade(self, symbol):
        history = self.market.get_history_trade(symbol=symbol, size=2000)
        datas = history.dict_data["data"]
        result = []
        [result.extend(data["data"]) for data in datas]
        self.tradeDB.insert_list(self.replace_key(result, symbol=symbol))

    def insert_kline(self, symbol):
        history = HistoryDownload(
            data_type="klines",
            start_date=datetime.datetime(2021, 5, 21),
            end_date=datetime.datetime(2021, 5, 23),
            all_period=["1min", "5min", "15min", "30min", "60min", "4hour", "1day"],
            all_symbols=[symbol],
            download_dir="/root/workspace/tmp/data",
        )
        history.b_download()

        self.kline_1min.insert_list(
            self.replace_key(
                self.market.get_candlestick(symbol=symbol, period="1min", size=2000).dict_data["data"], symbol
            )
        )
        self.kline_5min.insert_list(
            self.replace_key(
                self.market.get_candlestick(symbol=symbol, period="5min", size=2000).dict_data["data"], symbol
            )
        )
        self.kline_15min.insert_list(
            self.replace_key(
                self.market.get_candlestick(symbol=symbol, period="15min", size=2000).dict_data["data"], symbol
            )
        )
        self.kline_30min.insert_list(
            self.replace_key(
                self.market.get_candlestick(symbol=symbol, period="30min", size=2000).dict_data["data"], symbol
            )
        )
        self.kline_60min.insert_list(
            self.replace_key(
                self.market.get_candlestick(symbol=symbol, period="60min", size=2000).dict_data["data"], symbol
            )
        )
        self.kline_4hour.insert_list(
            self.replace_key(
                self.market.get_candlestick(symbol=symbol, period="4hour", size=2000).dict_data["data"], symbol
            )
        )
        self.kline_1day.insert_list(
            self.replace_key(
                self.market.get_candlestick(symbol=symbol, period="1day", size=2000).dict_data["data"], symbol
            )
        )

    def auto_save(self):
        self.symbolDB.auto_save()
        self.tradeDB.auto_save()
        self.kline_1min.auto_save()
        self.kline_5min.auto_save()
        self.kline_15min.auto_save()
        self.kline_30min.auto_save()
        self.kline_60min.auto_save()
        self.kline_4hour.auto_save()
        self.kline_1day.auto_save()
        self.kline_1mon.auto_save()
        self.kline_1week.auto_save()
        self.kline_1year.auto_save()

    def insert_data(self, trade=True, kline=True):
        symbols = self.symbolDB.select("select symbol from {} where state ='online'".format(self.symbolDB.table_name))
        for symbol in tqdm(symbols):
            try:
                symbol = symbol["symbol"]
                if trade:
                    self.insert_trade(symbol)
                if kline:
                    self.insert_kline(symbol)
                time.sleep(0.3)
            except Exception as e:
                print(e)

        print(self.symbolDB.select("select count(1) as num from " + self.symbolDB.table_name))
        print(self.tradeDB.select("select count(1) as num from " + self.tradeDB.table_name))
        print("1min\t{}".format(self.kline_1min.select("select count(1) as num from " + self.kline_1min.table_name)))
        print("5min\t{}".format(self.kline_5min.select("select count(1) as num from " + self.kline_5min.table_name)))
        print("15min\t{}".format(self.kline_15min.select("select count(1) as num from " + self.kline_15min.table_name)))
        print("30min\t{}".format(self.kline_30min.select("select count(1) as num from " + self.kline_30min.table_name)))
        print("60min\t{}".format(self.kline_60min.select("select count(1) as num from " + self.kline_60min.table_name)))
        print("4hour\t{}".format(self.kline_4hour.select("select count(1) as num from " + self.kline_4hour.table_name)))
        print("1day\t{}".format(self.kline_1day.select("select count(1) as num from " + self.kline_1day.table_name)))
        print("1week\t{}".format(self.kline_1week.select("select count(1) as num from " + self.kline_1week.table_name)))
        print("1mon\t{}".format(self.kline_1mon.select("select count(1) as num from " + self.kline_1mon.table_name)))
        print("1year\t{}".format(self.kline_1year.select("select count(1) as num from " + self.kline_1year.table_name)))

    def run(self):
        t = 0

        while True:
            self.insert_symbol()
            if t % 100 == 0:
                self.insert_data(trade=False, kline=True)
                self.auto_save()
            else:
                self.insert_data(trade=False, kline=False)
                time.sleep(120)
            t += 1

    def test(self):
        import pandas as pd

        condition = "id>=1619400078439 and id<=1619452799999 limit 111"
        sql = "select * from {} where {}".format(self.kline_1day.table_name, condition)
        print(sql)
        result = pd.read_sql(sql=sql, con=self.kline_1day.conn)
        print(result.head(10))


class TradeUpdate:
    def __init__(self, db_path="/root/workspace/tmp/coin/huobi.db"):
        api_key = read_secret(cate1="coin", cate2="huobi", cate3="api_key")
        secret_key = read_secret(cate1="coin", cate2="huobi", cate3="secret_key")
        self.market = MarketClient(api_key=api_key, secret_key=secret_key)
        self.generic = GenericClient(api_key=api_key, secret_key=secret_key)

        self.symbolDB = SymbolInfo(db_path=db_path)
        self.tradeDB = TradeDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_1min = Kline1MinDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_5min = Kline5MinDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_15min = Kline15MinDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_30min = Kline30MinDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_60min = Kline60MinDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_4hour = Kline4HourDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_1day = Kline1DayDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_1mon = Kline1MonDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_1week = Kline1WeekDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_1year = Kline1YearDetail(db_path=db_path, conn=self.symbolDB.conn)

        self.init()

    def init(self):
        self.symbolDB.create()
        self.tradeDB.create()
        self.kline_1min.create()
        self.kline_5min.create()
        self.kline_15min.create()
        self.kline_30min.create()
        self.kline_60min.create()
        self.kline_4hour.create()
        self.kline_1day.create()
        self.kline_1mon.create()
        self.kline_1week.create()
        self.kline_1year.create()

    @staticmethod
    def replace_key(data_dicts: list, symbol=None):
        res = []
        for _data_dict in data_dicts:
            data_dict = {"symbol": symbol}
            for key in _data_dict.keys():
                data_dict[key.replace("-", "_")] = _data_dict[key]
            res.append(data_dict)
        return res

    def insert_symbol(self):
        self.symbolDB.insert_list(self.replace_key(self.generic.get_exchange_symbols().data))

    def insert_trade(self, symbol):
        history = self.market.get_history_trade(symbol=symbol, size=2000)
        datas = history.dict_data["data"]
        result = []
        [result.extend(data["data"]) for data in datas]
        self.tradeDB.insert_list(self.replace_key(result, symbol=symbol))

    def insert_kline(self, symbol):
        self.kline_1min.insert_list(
            self.replace_key(
                self.market.get_candlestick(symbol=symbol, period="1min", size=2000).dict_data["data"], symbol
            )
        )
        self.kline_5min.insert_list(
            self.replace_key(
                self.market.get_candlestick(symbol=symbol, period="5min", size=2000).dict_data["data"], symbol
            )
        )
        self.kline_15min.insert_list(
            self.replace_key(
                self.market.get_candlestick(symbol=symbol, period="15min", size=2000).dict_data["data"], symbol
            )
        )
        self.kline_30min.insert_list(
            self.replace_key(
                self.market.get_candlestick(symbol=symbol, period="30min", size=2000).dict_data["data"], symbol
            )
        )
        self.kline_60min.insert_list(
            self.replace_key(
                self.market.get_candlestick(symbol=symbol, period="60min", size=2000).dict_data["data"], symbol
            )
        )
        self.kline_4hour.insert_list(
            self.replace_key(
                self.market.get_candlestick(symbol=symbol, period="4hour", size=2000).dict_data["data"], symbol
            )
        )
        self.kline_1day.insert_list(
            self.replace_key(
                self.market.get_candlestick(symbol=symbol, period="1day", size=2000).dict_data["data"], symbol
            )
        )
        self.kline_1mon.insert_list(
            self.replace_key(
                self.market.get_candlestick(symbol=symbol, period="1mon", size=2000).dict_data["data"], symbol
            )
        )
        self.kline_1week.insert_list(
            self.replace_key(
                self.market.get_candlestick(symbol=symbol, period="1week", size=2000).dict_data["data"], symbol
            )
        )
        self.kline_1year.insert_list(
            self.replace_key(
                self.market.get_candlestick(symbol=symbol, period="1year", size=2000).dict_data["data"], symbol
            )
        )

    def auto_save(self):
        self.symbolDB.auto_save()
        self.tradeDB.auto_save()
        self.kline_1min.auto_save()
        self.kline_5min.auto_save()
        self.kline_15min.auto_save()
        self.kline_30min.auto_save()
        self.kline_60min.auto_save()
        self.kline_4hour.auto_save()
        self.kline_1day.auto_save()
        self.kline_1mon.auto_save()
        self.kline_1week.auto_save()
        self.kline_1year.auto_save()

    def insert_data(self, trade=True, kline=True):
        symbols = self.symbolDB.select("select symbol from {} where state ='online'".format(self.symbolDB.table_name))
        for symbol in tqdm(symbols):
            try:
                symbol = symbol["symbol"]
                if trade:
                    self.insert_trade(symbol)
                if kline:
                    self.insert_kline(symbol)
                time.sleep(0.3)
            except Exception as e:
                print(e)

        print(self.symbolDB.select("select count(1) as num from " + self.symbolDB.table_name))
        print(self.tradeDB.select("select count(1) as num from " + self.tradeDB.table_name))
        print("1min\t{}".format(self.kline_1min.select("select count(1) as num from " + self.kline_1min.table_name)))
        print("5min\t{}".format(self.kline_5min.select("select count(1) as num from " + self.kline_5min.table_name)))
        print("15min\t{}".format(self.kline_15min.select("select count(1) as num from " + self.kline_15min.table_name)))
        print("30min\t{}".format(self.kline_30min.select("select count(1) as num from " + self.kline_30min.table_name)))
        print("60min\t{}".format(self.kline_60min.select("select count(1) as num from " + self.kline_60min.table_name)))
        print("4hour\t{}".format(self.kline_4hour.select("select count(1) as num from " + self.kline_4hour.table_name)))
        print("1day\t{}".format(self.kline_1day.select("select count(1) as num from " + self.kline_1day.table_name)))
        print("1week\t{}".format(self.kline_1week.select("select count(1) as num from " + self.kline_1week.table_name)))
        print("1mon\t{}".format(self.kline_1mon.select("select count(1) as num from " + self.kline_1mon.table_name)))
        print("1year\t{}".format(self.kline_1year.select("select count(1) as num from " + self.kline_1year.table_name)))

    def run(self):
        t = 0

        while True:
            self.insert_symbol()
            if t % 100 == 0:
                self.insert_data(trade=False, kline=True)
                self.auto_save()
            else:
                self.insert_data(trade=False, kline=False)
                time.sleep(120)
            t += 1

    def test(self):
        import pandas as pd

        condition = "id>=1619400078439 and id<=1619452799999 limit 111"
        sql = "select * from {} where {}".format(self.kline_1day.table_name, condition)
        print(sql)
        result = pd.read_sql(sql=sql, con=self.kline_1day.conn)
        print(result.head(10))


update = TradeUpdate()
update.run()
# update.test()

# ps -elf|grep update
# nohup /root/anaconda3/bin/python3.8 /root/workspace/fundata/funcoin/funcoin/huobi/dataset/update.py >>/root/workspace/tmp/funcoin-run-$(date +%Y-%m-%d).log 2>&1 &
# 2509
# sqlite3 -header -csv huobi.db "select * from trade_detail;" > trade_detail.csv



from datetime import datetime, timedelta

import ccxt
import pandas as pd
from funcoin.base.db import BaseTable
from funsecret import read_secret
from sqlalchemy import DOUBLE


class BaseTask(BaseTable):
    def __init__(self, exchange=None, *args, **kwargs):
        self.exchange = exchange or ccxt.binance(
            {
                "apiKey": read_secret("coin", "binance", "api_key"),
                "secret": read_secret("coin", "binance", "secret_key"),
            }
        )
        super().__init__(MyTable=None, *args, **kwargs)
        self.table_name = "base"

    def load(self):
        return pd.read_sql(sql=f"select * from {self.table_name}", con=self.engine.connect())

    def current_price(self, symbol="ETH/USDT"):
        data = self.exchange.fetch_ohlcv(symbol, since=int((datetime.now() + timedelta(minutes=-1)).timestamp() * 1000))
        if len(data) > 0:
            return data[-1][2]
        else:
            return 0


class AccountTask(BaseTask):
    table_name = "account"

    def __init__(self, *args, **kwargs):
        super(AccountTask, self).__init__(*args, **kwargs)

    def refresh(self, conn=None):
        data = self.exchange.fetch_balance()
        df = pd.DataFrame([data["total"], data["free"], data["used"]]).transpose()
        df.columns = ["total", "free", "used"]
        account = df.sort_values(["total"], ascending=False).reset_index()
        account.columns = ["symbol", "total", "free", "used"]

        def fun(data):
            if data["symbol"] == "BUSD":
                return 0
            if data["total"] == 0:
                return 0
            return self.current_price(f"{data['symbol']}/BUSD")

        account["price"] = account[["symbol", "total"]].apply(fun, axis=1)
        account.to_sql(name=self.table_name, con=conn or self.engine.connect(), if_exists="replace")


class MarketTask(BaseTask):
    table_name = "market"

    def __init__(self, *args, **kwargs):
        super(MarketTask, self).__init__(*args, **kwargs)

    def refresh(self, conn=None):
        self.exchange.load_markets()
        df = pd.DataFrame([dict((k, str(v)) for k, v in symbol.items()) for symbol in self.exchange.markets.values()])
        df.to_sql(name=self.table_name, con=conn or self.engine.connect(), if_exists="replace")


class Ticker24HTask(BaseTask):
    table_name = "ticker24h"

    def __init__(self, *args, **kwargs):
        super(Ticker24HTask, self).__init__(*args, **kwargs)

    def refresh(self, conn=None):
        param = {"type": "MINI"}
        df = pd.DataFrame(self.exchange.public_get_ticker_24hr(param))
        dtype = {"volume": DOUBLE, "quoteVolume": DOUBLE}
        df.to_sql(name=self.table_name, con=conn or self.engine.connect(), if_exists="replace", dtype=dtype)
